
import java.io.*;
import java.util.*;


/**
 *  File Utility Class
 */

public class FileUtil {


    /**
     * FileUtil() Constructor
     *
     * Don't let anyone instantiate this class
     */

    private FileUtil() {
    }


    /**
     * getFile() Method
     *
     * @return        File
     */

    public static File getFile(String f) {
        Properties fileProp = loadProperties(f);
        File file = new File(fileProp.getProperty("FileName"));
        return file;
    }


    /**
      * loadProperties Method
      *
      * @parm          String				Property fileName
      * @return        Properties
      */

    public static Properties loadProperties(String f) {
        Properties prop = new Properties();

        try {
            FileInputStream fs = getFileInputStream(f);
            prop.load(fs);
            fs.close();
        }
        catch (IOException io) {
            io.printStackTrace();
        }

        return prop;
    }


    /**
     * getFileInputStream() Method
     *
     * @parm          File			 fileName
     * @return        FileInputStream
     */

    public static FileInputStream getFileInputStream(String f) {
        FileInputStream fis = null;

        try {
            fis =  new FileInputStream(f);
        }
        catch (FileNotFoundException fnf) {
            System.out.println("\n File not found");
        }
        catch (IOException io) {
            io.printStackTrace();
        }

        return fis;
    }


    /**
     * openInputFile() Method
     *
     * @parm          File			fileName
     * @return        BufferedReader
     */

    public static BufferedReader openInputFile(File f) {
        BufferedReader br = null;

        try {
            br =  new BufferedReader(new FileReader(f));
        }
        catch (FileNotFoundException fnf) {
            System.out.println("\n File not found");
        }
        catch (IOException io) {
            io.printStackTrace();
        }

        return br;
    }


    /**
     * openOutputFile() Method
     *
     * @parm          File			fileName
     * @return        BufferedWriter
     */

    public static BufferedWriter openOutputFile(File f) {
        BufferedWriter bw = null;

        try {
            bw =  new BufferedWriter(new FileWriter(f));
        }
        catch (FileNotFoundException fnf) {
            System.out.println("\n File not found");
        }
        catch (IOException io) {
            io.printStackTrace();
        }

        return bw;
    }



    /**
     * close() Method
     *
     * @parm          BufferedReader
     * @return        void
     */

    public static void close(BufferedReader bw) {

        try {
            bw.close();
        }
        catch(IOException io) {
        }

    }



    /**
     * close() Method
     *
     * @parm          BufferedWriter
     * @return        void
     */

    public static void close(BufferedWriter bw) {

        try {
            bw.close();
        }
        catch(IOException io) {
        }

    }

}
