/*
 * $Id: InventoryEJB.java,v 1.11 2000/10/05 04:38:01 inder Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 */

package com.sun.j2ee.blueprints.inventory.ejb;


import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.ejb.DuplicateKeyException;
import com.sun.j2ee.blueprints.inventory.ejb.InventoryDAOException;
import com.sun.j2ee.blueprints.inventory.model.InventoryModel;

/**
 * Implementation of inventory as an Entity Bean
 */
public class InventoryEJB implements EntityBean {

    private String itemId; //primary key
    private int qty;

    private InventoryDAO inventoryDao;
    private EntityContext context;

    public InventoryEJB() {}

    public void ejbRemove() throws RemoveException {

        try{
            InventoryDAO dao = getDAO();
            dao.setItemId(this.itemId);
            dao.remove();
        } catch (InventoryDAOException se) {
            context.setRollbackOnly();
            throw new RemoveException(se.getMessage());
        }
    }

    public void setEntityContext(EntityContext ec) {
        context = ec;
    }

    public void ejbLoad() {
        InventoryDAO dao;
        try{
            dao = getDAO();
            dao.setItemId(this.itemId);
            dao.load();
        } catch (InventoryDAOException se) {
            throw new EJBException (se.getMessage());
        }
        itemId = dao.getItemId();
        qty = dao.getQuantity();

    }

    public void ejbStore() {
        try{
            InventoryDAO dao = getDAO();
            dao.setItemId(this.itemId);
            dao.setQuantity(qty);
            dao.store();
        } catch (InventoryDAOException se) {
            throw new EJBException (se.getMessage());
        }
    }

    public String ejbFindByPrimaryKey (String key) throws FinderException {
        try{
            InventoryDAO dao = getDAO();
            dao.setItemId(key);
            return(dao.findByPrimaryKey());
        } catch (InventoryDAOException se) {
            throw new FinderException (se.getMessage());
        }
    }

    public void unsetEntityContext() {}

    public void ejbActivate() {
      this.itemId = (String)context.getPrimaryKey();
    }

    public void ejbPassivate() {
      this.itemId       = null;
      this.inventoryDao = null;
    }

    // business methods

    /**
     * @return  the InventoryModel containing the
     * inventory data details
     */
    public InventoryModel getDetails() {
        return(new InventoryModel(itemId,qty));
    }
    /**
     * reduce the current qty of an item in inventory.
     */
    public void reduceQuantity( int qty) {
        this.qty -= qty;
    }

    /**
     * increase the current qty of an item in inventory.
     */
    public void addQuantity(int qty) {
        this.qty += qty;
    }

    private InventoryDAO getDAO() throws InventoryDAOException {
        if(inventoryDao == null) {
            inventoryDao = new InventoryDAO();
        }
        return inventoryDao;
    }

}
