/*
 * $Id: OrderHandler.java,v 1.14 2000/10/09 02:00:20 gmurray Exp $
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * Copyright 2000 Sun Microsystems, Inc. Tous droits rservs.
 *
 */

package com.sun.j2ee.blueprints.petstore.control.ejb;


import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.AbstractList;
import java.util.Collection;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Locale;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;

import com.sun.j2ee.blueprints.util.tracer.Debug;
import com.sun.j2ee.blueprints.petstore.util.JNDIUtil;
import com.sun.j2ee.blueprints.petstore.util.EJBUtil;

import com.sun.j2ee.blueprints.inventory.ejb.InventoryHome;
import com.sun.j2ee.blueprints.inventory.ejb.Inventory;

// order imports

import com.sun.j2ee.blueprints.customer.customer.ejb.Customer;
import com.sun.j2ee.blueprints.customer.customer.ejb.CustomerHome;
import com.sun.j2ee.blueprints.customer.order.model.LineItem;

// cart imports

import com.sun.j2ee.blueprints.shoppingcart.cart.ejb.ShoppingCart;
import com.sun.j2ee.blueprints.shoppingcart.cart.model.ShoppingCartModel;
import com.sun.j2ee.blueprints.shoppingcart.cart.model.CartItem;

// email imports
import com.sun.j2ee.blueprints.mail.ejb.Mailer;

import com.sun.j2ee.blueprints.petstore.control.event.OrderEvent;
import com.sun.j2ee.blueprints.petstore.control.event.EStoreEvent;
import com.sun.j2ee.blueprints.petstore.control.EStoreEventException;


public class OrderHandler extends StateHandlerSupport {


  public void perform(EStoreEvent event) throws EStoreEventException {
        try {
            OrderEvent oe = (OrderEvent)event;
            ShoppingCart cart = machine.getShoppingClientControllerEJB().getShoppingCart();
            Locale locale = oe.getLocale();
            InventoryHome inventHome = EJBUtil.getInventoryHome();
            Iterator ci = ((ShoppingCartModel)cart.getDetails(locale)).getItems();
            ArrayList lineItems = new ArrayList();
            int lineNo = 0;
            double total = 0;
            while (ci.hasNext()) {
                lineNo++;
                CartItem cartItem = (CartItem) ci.next();
                LineItem li = new LineItem(cartItem.getItemId(),
                cartItem.getQuantity(),
                cartItem.getUnitCost(), lineNo);
                 lineItems.add(li);
                 total += (cartItem.getUnitCost() * (double) cartItem.getQuantity());
            }

            for (Iterator it = lineItems.iterator(); it.hasNext();){
                LineItem LI = (LineItem)it.next();
                Inventory inventRef = inventHome.findByPrimaryKey(LI.getItemNo());
                inventRef.reduceQuantity(LI.getQty());
            }

            CustomerHome home = EJBUtil.getCustomerHome();
            Customer cust = home.create();
            int order = cust.createOrder(lineItems,
                                oe.getShippingAddress(),
                                oe.getBillingAddress(),
                                oe.getShipToFirstName(),
                                oe.getShipToLastName(),
                                oe.getBillToFirstName(),
                                oe.getBillToLastName(),
                                oe.getCreditCard(),
                                oe.getCarrier(), total);
            // put the requestId and the orderId in a table to match up later
            HashMap orderTable = (HashMap)machine.getAttribute("orderTable");
            if (orderTable == null) {
                orderTable = new HashMap();
                machine.setAttribute("orderTable", orderTable);
            }
            orderTable.put(oe.getRequestId() + "", order +"");

            // empty shopping cart
            cart.empty();

            if (JNDIUtil.sendConfirmationMail()) {
                // send order confirmation mail.

                MailAction mAction = new MailAction();
                mAction.sendConfirmationMessage(order);
            }
        } catch (DuplicateKeyException dke) {
        // How can another order have the same id ?
            throw new EJBException(dke);
        } catch (CreateException ce) {
            throw new EJBException(ce);
        } catch (FinderException fe) {
            throw new EJBException(fe);
        } catch (Exception ex) {
            Debug.println("OrderHandler caught:" + ex);
        }
    }
}
