import java.sql.SQLException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.DriverManager;

import java.util.Vector;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.util.StringTokenizer;


public class PopulateTables {

    private static final String databaseDriver = "RmiJdbc.RJDriver";
    private static final String user= "estoreuser";
    private static final String password = "estore";

    public static  Connection getDirectConnection(String host) {
        String databaseName = "jdbc:rmi://" + host + ":1099/jdbc:cloudscape:CloudscapeDB;autocommit=true";
        try {
            Class.forName(databaseDriver);
        } catch (ClassNotFoundException e) {
                System.out.println("PopulateTables: caught getting connection: " + e);
        }
        Connection con = null;
        try {
            con = DriverManager.getConnection(databaseName, user, password);
        } catch (java.sql.SQLException se) {
                System.out.println("PopulateTables: caught getting connection: " + se);
        }
        return con;
    }

    public static String loadFile(String fileName, String encoding){
        FileInputStream in;
        int filesize=0;
        String returnString = new String("");
        byte [] inData = null;
        try {
                File f = new File (fileName);
                in = new FileInputStream(fileName);
                filesize = (int) f.length();
                inData = new byte[filesize];
                in.read (inData, 0, filesize);
                in.close();

        }catch (IOException e){
                System.out.print ("Error Reading file: " + e + "\n");
                System.exit(0);
        }catch (Exception e){
            System.out.print ("Error Reading: " + e + "\n");
            System.exit(0);
        }
        try{
        returnString = new String(inData,0, filesize, encoding);
        }catch (UnsupportedEncodingException enex){
            System.out.println("Unable to Convert Source File");
            System.exit(0);
        }
    return returnString;
    }

    public static void writeFile(String fileName, String encoding, String data){
        FileOutputStream out;
        try {
                File f = new File (fileName);
                if (f.exists()) f.delete();
                out = new FileOutputStream(fileName);
                OutputStreamWriter writer = new OutputStreamWriter(out, encoding);
                int fileSize = data.length();
                writer.write (data, 0, fileSize);
                writer.close();
                out.close();
        }catch (IOException e){
                System.out.print ("Error Writing file: " + e + "\n");
                System.exit(0);
        }catch (Exception e){
                System.out.print ("Error Writing file: " + e +"\n");
                System.exit(0);
        }
    }// end writeFile

    public static void populateTables(String fileName, String encoding, String host) {
        System.out.println("Loading " + fileName);
        String text = loadFile(fileName, encoding);
        //writeFile("out.txt", "UTF8", text);
        System.out.println("Connecting to " + host);
        Connection con = getDirectConnection(host);
        System.out.println("Connection is " + con);

        StringTokenizer strTok = new StringTokenizer(text, "\n");
        while (strTok.hasMoreTokens()) {
            String querry = strTok.nextToken().trim();
            if (!querry.equals("") && !querry.equals("?")) {
               System.out.println("Querry=" + querry);
                try {

                    Vector targetStrings = new Vector();
                    String processedQuerry = "";
                    String VALUES = "VALUES";
                    int startIndex = -1;
                    // get working string
                    int valueStart = querry.toUpperCase().indexOf(VALUES);
                    if (valueStart != -1) {
                           startIndex = querry.indexOf("(", valueStart + VALUES.length());
                           if (startIndex < querry.length()) processedQuerry = querry.substring(0, startIndex);
                    }
                    // find all the target strings
                    if (startIndex != -1) {
                        int index = startIndex;
                        int literalStart = -1;
                        while (index < querry.length()) {
                            if (querry.charAt(index) == '\'') {
                                if (literalStart == -1 && index + 1 < querry.length()) {
                                    literalStart = index +1;
                                } else {
                                    String targetString = querry.substring(literalStart, index);
                                    targetStrings.addElement(targetString);
                                    literalStart = -1;
                                    processedQuerry += "?";
                                    if (index +1 < querry.length()) index++;
                                }
                            }
                            if (literalStart == -1 && querry.charAt(index) == ')') {
                                break;
                            } else if (literalStart == -1) {
                                processedQuerry += querry.charAt(index);
                            }
                            index++;

                        }
                        processedQuerry += ")";
                        System.out.println("Processed Querry=" + processedQuerry);
                        if (con != null) {
                            PreparedStatement stmt = con.prepareStatement(processedQuerry);
                            for (int loop=0; loop < targetStrings.size(); loop++) {
                                String arg = (String)targetStrings.elementAt(loop);
                                stmt.setString(loop +1, arg);
                           }
                           int resultCount = stmt.executeUpdate();
                           stmt.close();
                           System.out.println(resultCount + " row(s) updated/inserted/deleted");
                       }
                    } else {
                        if (con != null) {
                            PreparedStatement stmt = con.prepareStatement(querry);
                            int resultCount = stmt.executeUpdate();
                            stmt.close();
                            System.out.println(resultCount + " row(s) updated/inserted/deleted");
                       }

                    }
               } catch (SQLException ex) {
                   System.out.println("Caught : " + ex);
               }
            }
        }
    }

    public static void main(String[] args) {
        if (args.length >= 2) {
            populateTables(args[0], args[1], args[2]);
        } else {
            System.out.println("usage: populate <sql file> <sql_file_encoding> <host>");

        }
    }
}
